/**********************************************************************
* File: Adc.c -- File for Labs 6, 7, 8, 9, and 10 (not used in Lab 5)
* Devices: TMS320F2812, TMS320F2811, TMS320F2810
* Author: Technical Training Organization (TTO), Texas Instruments
* History:
*   11/10/03 - original (based on DSP281x header files v1.00)
**********************************************************************/

#include "DSP281x_Device.h"
#include "lab.h"


void DelayUs(volatile Uint16);			//function prototype


/**********************************************************************
* Function: InitAdc()
*
* Description: Initializes the ADC on the F2812
**********************************************************************/
void InitAdc(void)
{

/*** Reset the ADC module ***/
	AdcRegs.ADCTRL1.bit.RESET = 1;		// Reset the ADC module
	asm(" RPT #10 || NOP");				// Must wait 12-cycles (worst-case) for ADC reset to take effect
	
/*** Must follow the proper ADC power-up sequence ***/
	AdcRegs.ADCTRL3.all = 0x00C8;		// first power-up ref and bandgap circuits
/*
 bit 15-8      0's:    reserved
 bit 7         1:      ADCRFDN, reference power, 1=power on
 bit 6         1:      ADCBGDN, bandgap power, 1=power on
 bit 5         0:      ADCPWDN, main ADC power, 1=power on
 bit 4-1       0100:   ADCCLKPS, clock prescaler, ADCLK=HSPCLK/(2*ADCCLKPS)
 bit 0         0:      SMODE_SEL, 0=sequential sampling, 1=simultaneous sampling
*/

	DelayUs(10000);						// Wait 10ms before setting ADCPWDN
	AdcRegs.ADCTRL3.bit.ADCPWDN = 1;	// Set ADCPWDN=1 to power main ADC
	DelayUs(1000);						// Wait 1ms before using the ADC

/*** Configure the other ADC register ***/
	AdcRegs.ADCMAXCONV.all = 0x????;
/*
 bit 15-7      0's:    reserved
 bit 6-4       ???:    MAX_CONV2 value
 bit 3-0       ????:   MAX_CONV1 value (0 means 1 conversion)
*/

	AdcRegs.ADCCHSELSEQ1.bit.CONV00 = ?;	// Convert Channel ?

	AdcRegs.ADCTRL1.all = 0x????;
/*
 bit 15        0:      reserved
 bit 14        ?:      RESET
 bit 13-12     10:     SUSMOD, 10=stop after current conversion on emulator suspend
 bit 11-8      ????:   ACQ_PS (Acquisition)
 bit 7         ?:      CPS (Core clock)
 bit 6         ?:      CONT_RUN
 bit 5         0:      SEQ_OVRD, 0=disabled, 1=enabled
 bit 4         ?:      SEQ_CASC
 bit 3-0       0000:   reserved
*/

	AdcRegs.ADCTRL2.all = 0x????;
/*
 bit 15        ?:      EVB_SOC_SEQ
 bit 14        ?:      RST_SEQ1
 bit 13        ?:      SOC_SEQ1
 bit 12        0:      reserved
 bit 11        ?:      INT_ENA_SEQ1
 bit 10        ?:      INT_MOD_SEQ1
 bit 9         0:      reserved
 bit 8         1:      EVA_SOC_SEQ1, 1=SEQ1 start from EVA
 bit 7         0:      EXT_SOC_SEQ1, 1=SEQ1 start from ADCSOC pin
 bit 6         ?:      RST_SEQ2
 bit 5         ?:      SOC_SEQ2
 bit 4         0:      reserved
 bit 3         ?:      INT_ENA_SEQ2
 bit 2         ?:      INT_MOD_SEQ2
 bit 1         0:      reserved
 bit 0         ?:      EVB_SOC_SEQ2
*/

/*** Enable the ADC interrupt ***/
	                                 	// Enable ADCINT in PIE group 1
	             						// Enable INT1 in IER to enable PIE group 1

} // end AdcInit()


/*********************************************************************
* Function: DelayUs()                                                *
* Description: Implements a time delay.                              *
* DSP: TMS320F2812                                                   *
* Last Modified: 06/28/02                                            *
* Include files: none                                                *
* Function Prototype: void DelayUs(volatile Uint16)                  *
* Useage: DelayUs(Usec);                                             *
* Input Parameters: Uint16 Usec = time delay in microseconds         *
* Return Value: none                                                 *
* Notes: The execution time of this routine is rough, based upon a   *
*        150MHz CPUCLK.  It has been tested using all optimization   *
*        levels of the compiler to give approximately a 1us inner    *
*        loop.                                                       *   
*********************************************************************/
void DelayUs( volatile Uint16 Usec )
{
    while( Usec-- )					// 1us loop at 150MHz CPUCLK
	{
		asm(" RPT #139 || NOP");
	}

} // end DelayUs()


/*** end of file *****************************************************/
